//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "UCifraRC4.h"

//---------------------------------------------------------------------------

#pragma package(smart_init)

void inicial(CifraRC4& RC); // alfabeto inicial de partida para generar la semilla
void siguiente(CifraRC4& RC); // siguiente alfabeto para la generacin de la secuencia
CharExt genABC(CifraRC4& RC); // siguiente de la secuencia
CharExt addCfr(CharExt a, CharExt b);
CharExt difCfr(CharExt a, CharExt b);

bool init(CifraRC4& RC)
{
        RC.cA = RC.cB = 0;
        init(RC.abc, LonAlfa);
        init(RC.clave);
        RC.funCfr = addCfr;
        return true;
}

void free(CifraRC4& RC)
{
        RC.cA = RC.cB = 0;
        free(RC.abc);
        free(RC.clave);
}

bool setClave(CifraRC4& RC, const CadenaExt& clave) // semilla
{
        inicial(RC);

        if(size(clave) == 0)
        {
                return false;
        }
        else
        {
                assign(RC.clave, clave);
                for(RC.cA = 0; RC.cA < LonAlfa; RC.cA++)
                {
                        RC.cB = (RC.cB + at(RC.abc, RC.cA) + at(clave, RC.cA % size(clave))) % LonAlfa;
                        swap(RC.abc, RC.cA, RC.cB);
                }
                return true;
        }
}

void inicial(CifraRC4& RC) // alfabeto inicial de partida para generar la semilla
{
        clear(RC.abc);
        resize(RC.abc, LonAlfa);

        for(int i = 0; i < LonAlfa; i++)
        {
                push_back(RC.abc, CharExt(i));
        }

        RC.cA = RC.cB = 0;
}

void siguiente(CifraRC4& RC) // siguiente alfabeto para la generacin de la secuencia
{
        RC.cA++;
        RC.cA %= LonAlfa;
        RC.cB += at(RC.abc, RC.cA);
        RC.cB %= LonAlfa;
        swap(RC.abc, RC.cA, RC.cB);
}

CharExt genABC(CifraRC4& RC) // siguiente de la secuencia
{
        siguiente(RC);

        return at(RC.abc, (at(RC.abc, RC.cA) + at(RC.abc, RC.cB)) % LonAlfa);
}

void setOperCfr(CifraRC4& RC, opeCfr op)
{
        if(op == cifrar)
        {
                RC.funCfr = addCfr;
        }
        else
        {
                RC.funCfr = difCfr;
        }
}


CharExt addCfr(CharExt a, CharExt b)
{
        return (a + b) % LonAlfa;
}

CharExt difCfr(CharExt a, CharExt b)
{
        return (a + LonAlfa - b) % LonAlfa;
}

void exeOperCfr(CifraRC4& RC, const CadenaExt& Ent, CadenaExt& Sal)
{
        CharExt e, r, s;
        clear(Sal);
        resize(Sal, size(Ent));
        for(tSize i = 0; i < size(Ent); i++)
        {
                e = at(Ent, i);
                if(e >= LonCtrl)
                {
                        e -= LonCtrl;
                        r = genABC(RC);
                        s = RC.funCfr(e, r);
                        s += LonCtrl;
                }
                else
                {
                        s = e;
                }
                push_back(Sal, s);
        }
}

